@@ -17,7 +17,10 @@ import uuid
 import warnings
 
 from datetime import datetime
+from inspect import isclass
+from random import Random
 from types import FunctionType
+from threading import local
 
 if sys.version_info >= (3, 2):
     import contextlib
@@ -34,10 +37,10 @@ from raven.conf import defaults
 from raven.conf.remote import RemoteConfig
 from raven.exceptions import APIError, RateLimited
 from raven.utils import json, get_versions, get_auth_header, merge_dicts
-from raven._compat import text_type, iteritems
+from raven.utils.compat import text_type, iteritems
 from raven.utils.encoding import to_unicode
 from raven.utils.serializer import transform
-from raven.utils.stacks import get_stack_info, iter_stack_frames, slim_string
+from raven.utils.stacks import get_stack_info, iter_stack_frames
 from raven.utils.transaction import TransactionStack
 from raven.transport.registry import TransportRegistry, default_transports
 
@@ -59,6 +62,9 @@ SDK_VALUE = {
 # singleton for the client
 Raven = None
 
+if sys.version_info >= (3, 2):
+    basestring = str
+
 
 def get_excepthook_client():
     hook = sys.excepthook
@@ -145,11 +151,14 @@ class Client(object):
 
     def __init__(self, dsn=None, raise_send_errors=False, transport=None,
                  install_sys_hook=True, install_logging_hook=True,
-                 hook_libraries=None, enable_breadcrumbs=True, **options):
+                 hook_libraries=None, enable_breadcrumbs=True,
+                 _random_seed=None, **options):
         global Raven
 
         o = options
 
+        self._local_state = local()
+
         self.raise_send_errors = raise_send_errors
 
         # configure loggers first
@@ -182,20 +191,24 @@ class Client(object):
 
         context = o.get('context')
         if context is None:
-            context = {'sys.argv': sys.argv[:]}
+            context = {'sys.argv': getattr(sys, 'argv', [])[:]}
         self.extra = context
         self.tags = o.get('tags') or {}
         self.environment = o.get('environment') or None
         self.release = o.get('release') or os.environ.get('HEROKU_SLUG_COMMIT')
+        self.repos = self._format_repos(o.get('repos'))
+        self.sample_rate = (
+            o.get('sample_rate')
+            if o.get('sample_rate') is not None
+            else 1
+        )
         self.transaction = TransactionStack()
-        # find the root transaction as the command which launched this
-        # process
-        self.transaction.push(slim_string(' '.join(sys.argv), 128))
-
         self.ignore_exceptions = set(o.get('ignore_exceptions') or ())
 
         self.module_cache = ModuleProxyCache()
 
+        self._random = Random(_random_seed)
+
         if not self.is_enabled():
             self.logger.info(
                 'Raven is not configured (logging is disabled). Please see the'
@@ -220,6 +233,17 @@ class Client(object):
 
         self.hook_libraries(hook_libraries)
 
+    def _format_repos(self, value):
+        if not value:
+            return {}
+        result = {}
+        for path, config in iteritems(value):
+            if path[0] != '/':
+                # assume its a module
+                path = os.path.abspath(__import__(path).__file__)
+            result[path] = config
+        return result
+
     def set_dsn(self, dsn=None, transport=None):
         if not dsn and os.environ.get('SENTRY_DSN'):
             msg = "Configuring Raven from environment variable 'SENTRY_DSN'"
@@ -468,6 +492,7 @@ class Client(object):
         data.setdefault('event_id', event_id)
         data.setdefault('platform', PLATFORM_NAME)
         data.setdefault('sdk', SDK_VALUE)
+        data.setdefault('repos', self.repos)
 
         # insert breadcrumbs
         if self.enable_breadcrumbs:
@@ -541,7 +566,8 @@ class Client(object):
         })
 
     def capture(self, event_type, data=None, date=None, time_spent=None,
-                extra=None, stack=None, tags=None, **kwargs):
+                extra=None, stack=None, tags=None, sample_rate=None,
+                **kwargs):
         """
         Captures and processes an event and pipes it off to SentryClient.send.
 
@@ -589,6 +615,7 @@ class Client(object):
         :param extra: a dictionary of additional standard metadata
         :param stack: a stacktrace for the event
         :param tags: dict of extra tags
+        :param sample_rate: a float in the range [0, 1] to sample this message
         :return: a tuple with a 32-length string identifying this event
         """
 
@@ -610,7 +637,14 @@ class Client(object):
             event_type, data, date, time_spent, extra, stack, tags=tags,
             **kwargs)
 
-        self.send(**data)
+        # should this event be sampled?
+        if sample_rate is None:
+            sample_rate = self.sample_rate
+
+        if self._random.random() < sample_rate:
+            self.send(**data)
+
+        self._local_state.last_event_id = data['event_id']
 
         return data['event_id']
 
@@ -626,7 +660,7 @@ class Client(object):
             for frame in data['stacktrace']['frames']:
                 yield frame
         if 'exception' in data:
-            for frame in data['exception']['values'][0]['stacktrace']['frames']:
+            for frame in data['exception']['values'][-1]['stacktrace'].get('frames', []):
                 yield frame
 
     def _successful_send(self):
@@ -658,9 +692,9 @@ class Client(object):
         """
         message = data.pop('message', '<no message value>')
         output = [message]
-        if 'exception' in data and 'stacktrace' in data['exception']['values'][0]:
+        if 'exception' in data and 'stacktrace' in data['exception']['values'][-1]:
             # try to reconstruct a reasonable version of the exception
-            for frame in data['exception']['values'][0]['stacktrace']['frames']:
+            for frame in data['exception']['values'][-1]['stacktrace'].get('frames', []):
                 output.append('  File "%(fn)s", line %(lineno)s, in %(func)s' % {
                     'fn': frame.get('filename', 'unknown_filename'),
                     'lineno': frame.get('lineno', -1),
@@ -688,11 +722,11 @@ class Client(object):
 
         try:
             transport = self.remote.get_transport()
-            if transport.async:
-                transport.async_send(data, headers, self._successful_send,
+            if transport.is_async:
+                transport.async_send(url, data, headers, self._successful_send,
                                      failed_send)
             else:
-                transport.send(data, headers)
+                transport.send(url, data, headers)
                 self._successful_send()
         except Exception as e:
             if self.raise_send_errors:
@@ -787,12 +821,19 @@ class Client(object):
         exc_type = exc_info[0]
         exc_name = '%s.%s' % (exc_type.__module__, exc_type.__name__)
         exclusions = self.ignore_exceptions
+        string_exclusions = (e for e in exclusions if isinstance(e, basestring))
+        wildcard_exclusions = (e for e in string_exclusions if e.endswith('*'))
+        class_exclusions = (e for e in exclusions if isclass(e))
 
-        if exc_type.__name__ in exclusions:
+        if exc_type in exclusions:
+            return False
+        elif exc_type.__name__ in exclusions:
             return False
         elif exc_name in exclusions:
             return False
-        elif any(exc_name.startswith(e[:-1]) for e in exclusions if e.endswith('*')):
+        elif any(issubclass(exc_type, e) for e in class_exclusions):
+            return False
+        elif any(exc_name.startswith(e[:-1]) for e in wildcard_exclusions):
             return False
         return True
 
@@ -871,6 +912,14 @@ class Client(object):
 
     capture_breadcrumb = captureBreadcrumb
 
+    @property
+    def last_event_id(self):
+        return getattr(self._local_state, 'last_event_id', None)
+
+    @last_event_id.setter
+    def last_event_id(self, value):
+        self._local_state.last_event_id = value
+
 
 class DummyClient(Client):
     "Sends messages into an empty void"
